try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup
setup(
    name='icecreamy',
    version='0.0.6.2',
    author='xenups',
    include_package_data=True,
    author_email='xenups@gmail.com',
    packages=['ICECREAM', 'ICECREAM', 'ICECREAM.api_cache', 'ICECREAM.migration_tool', 'ICECREAM.models',
              'ICECREAM.statics', 'ICECREAM.assets'],
    url='https://github.com/xenups/ICECREAM',
    license='LICENSE.txt',
    scripts=['make_project.py'],
    description='restful framework with bottle',
    entry_points={
        'console_scripts': ['make_project=ICECREAM.makeproject:make_project'],
    },
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    install_requires=[
        "bottle-jwt2",
        "alembic",
        "appdirs",
        "bottle-jwt2",
        "bottle",
        "codecov",
        "coverage",
        "cryptography",
        "gevent",
        "greenlet",
        "gunicorn",
        "jdatetime",
        "Jinja2",
        "jsonschema",
        "marshmallow",
        "marshmallow-sqlalchemy",
        "mongosql",
        "nose",
        "psycopg2-binary",
        "PyJWT",
        "python-dateutil",
        "python-dotenv",
        "python-throttle",
        "ratelimit",
        "redis",
        "rootpath",
        "sentry-sdk",
        "simplerbac",
        "sqla-wrapper",
        "sqlacodegen",
        "SQLAlchemy==1.3.23",
        "sqlalchemy-filters",
        "SQLAlchemy-Searchable",
        "SQLAlchemy-Utils",
        "tornado",
        "tox",
        "virtualenv",
        "WebTest",
        "Werkzeug",
        "ujson",

    ],
)
